using MediatR;
using Sledgemapper.Api.Infrastructure.Data;
using Sledgemapper.Api.Commands;
using Sledgemapper.Api.Notifications;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;

namespace Sledgemapper.Api.Handlers
{
    public class DeleteWallCommandHandler : BaseCommandHandler<DeleteWallCommand, bool>
    {
        public DeleteWallCommandHandler(IMediator mediator, SledgemapperDbContext dbcontext) : base(mediator, dbcontext)
        {
        }

        public override async Task<bool> Handle(DeleteWallCommand command, CancellationToken cancellationToken)
        {
            await CheckAuthorization(command);
            var jsonString = JsonSerializer.Serialize(command.Wall);
            var session = await SaveLog(command, "D", "W", jsonString, cancellationToken);
            await Mediator.Publish(new DeleteWallNotification(session, command.Wall, command.UserId), cancellationToken);
            return true;
        }
    }
}
