using MediatR;
using Sledgemapper.Api.Commands;
using Sledgemapper.Api.Infrastructure.Data;
using Sledgemapper.Api.Notifications;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;

namespace Sledgemapper.Api.Handlers
{

    public class NewOverlayCommandHandler : BaseCommandHandler<NewOverlayCommand, bool>
    {
        public NewOverlayCommandHandler(IMediator mediator, SledgemapperDbContext dbcontext) : base(mediator, dbcontext)
        {
        }

        public override async Task<bool> Handle(NewOverlayCommand command, CancellationToken cancellationToken)
        {
            await CheckAuthorization(command);
            var jsonString = JsonSerializer.Serialize(command.Overlay);
            var session = await SaveLog(command, "N", "O", jsonString, cancellationToken);
            await Mediator.Publish(new NewOverlayNotification(session, command.Overlay, command.UserId), cancellationToken);
            return true;
        }
    }
}
