using Microsoft.AspNetCore.SignalR;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Collections.Concurrent;
using Sledgemapper.Shared.Entities;
using System;
using Sledgemapper.Api.Infrastructure.Data;
using Microsoft.AspNetCore.Authorization;
using Sledgemapper.Api.Core.Entities;
using Sledgemapper.Api.Models;
using Sledgemapper.Shared.Clients;

namespace Sledgemapper.Api.Hubs
{
    [Authorize]
    public class SledgemapperHub : Hub<ISledgemapperClient>
    {
        private static readonly ConcurrentDictionary<Guid, string> UserColors = new();
        private readonly SledgemapperDbContext _dbContext;

        public SledgemapperHub(SledgemapperDbContext dbContext/*, DataContext datacontext*/)
        {
            _dbContext = dbContext;
        }

        // other colors
        // #cca300, #20f200, #004011, #00e6d6, #005c73, #0057d9, #d900ca, #660029, #d9003a
        // private static Dictionary<string, Session> _sessions = new Dictionary<string, Session>();
        public List<string> Colors = new()
        {
            "#e6194B",
            "#f58231",
            "#3cb44b",
            "#000075",
            "#911eb4",
            "#800000",
            "#808000",
            "#469990"
        };

        public async Task NewTile(string sessionName, Tile tile)
        {
            await Clients.Group(sessionName).NewTile(tile);
        }

        public async Task NewRoom(string sessionName, Room room)
        {
            await Clients.Group(sessionName).NewRoom(room);
        }

        public async Task NewLine(string sessionName, Line line)
        {
            await Clients.Group(sessionName).NewLine(line);
        }

        public async Task NewWall(string sessionName, Wall tile)
        {
            await Clients.Group(sessionName).NewWall(tile);
        }

        public async Task NewOverlay(Guid mapId, Overlay tile)
        {
            await Clients.Group(mapId.ToString()).NewOverlay(tile);
        }

        public async Task NewNote(string sessionName, Note note)
        {
            await Clients.Group(sessionName).NewNote(note);
        }

        public async Task DeleteTile(string sessionName, Tile tile)
        {
            await Clients.Group(sessionName).DeleteTile(tile);
        }

        public async Task DeleteWall(string sessionName, Wall tile)
        {
            await Clients.Group(sessionName).DeleteWall(tile);
        }

        public async Task DeleteOverlay(string sessionName, Overlay tile)
        {
            await Clients.Group(sessionName).DeleteOverlay(tile);
        }

        public async Task Ping(string sessionName, Tile location)
        {
            var userId = new Guid(Context.User.Claims.FirstOrDefault(m => m.Type == "Id").Value);

            var user = _dbContext.Users.First(u => u.Id == userId.ToString());

            var player = new Player { UserId = userId, Initials = user.Initials, Position = new Tile { X = 0, Y = 0 }, Color = UserColors[userId] };
            await Clients.Group(sessionName).Ping(new Ping { X = location.X, Y = location.Y, Player = player });
        }

        public async Task<bool> JoinSession(Guid mapId)
        {
            var session = _dbContext.Sessions.FirstOrDefault(s => s.SessionId == mapId);

            var userId = new Guid(Context.User.Claims.FirstOrDefault(m => m.Type == "Id").Value);

            if (session != null)
            {
                var userSession = new SessionUser { SessionId = session.SessionId, UserId = userId };
                _dbContext.SessionUsers.Add(userSession);
                await _dbContext.SaveChangesAsync();
                await Groups.AddToGroupAsync(Context.ConnectionId, session.SessionId.ToString());
                var user = _dbContext.Users.First(u => u.Id == userId.ToString());

                var player = new Player { UserId = userId, Initials = user.Initials, Position = new Tile { X = 0, Y = 0 }, Color = UserColors[userId] };

                await Clients.Group(session.SessionId.ToString()).NewPlayer(player);
                await Clients.Group(session.SessionId.ToString()).RefreshPlayers();



                return true;
            }
            else
            {
                return false;
            }
        }

        //public async Task UpdatePosition(string sessionName, int sessionId, Tile tile)
        //{
        //    //var userId = int.Parse(Context.User.Identity.Name);
        //    //var SessionUsers = _dbContext.SessionUsers.Where(m => m.SessionId == sessionId).OrderBy(m => m.UserId).ToList();
        //    //var user = _dbContext.Users.First(u => u.Id == Context.User.Identity.Name);
        //    //var player = new Player { UserId = userId, Initials = user.Initials, Position = tile, Color = UserColors[userId] };
        //    //await Clients.Group(sessionName).PlayerUpdate(player);
        //}

        public async Task UpdatePosition(string sessionName, Guid sessionId, Tile tile)
        {
            var userId = new Guid(Context.User.Claims.FirstOrDefault(m => m.Type == "Id").Value);
            var sessionUsers = _dbContext.SessionUsers.Where(m => m.SessionId == sessionId).OrderBy(m => m.UserId).ToList();
            var user = _dbContext.Users.First(u => u.Id == userId.ToString());
            var player = new Player { UserId = userId, Initials = user.Initials, Position = tile, Color = UserColors[userId] };
            await Clients.Group(sessionId.ToString()).PlayerUpdate(player);
        }

        public override async Task OnConnectedAsync()
        {
            var userId = new Guid(Context.User.Claims.FirstOrDefault(m => m.Type == "Id").Value);
            var userConnection = new UserConnection { ConnectionId = Context.ConnectionId, UserId = userId };
            _dbContext.UserConnections.Add(userConnection);
            await _dbContext.SaveChangesAsync();
            var availableColor = Colors.First(m => !UserColors.Values.Contains(m));
            UserColors.AddOrUpdate(userId, availableColor, (key, oldValue) => availableColor);
            await base.OnConnectedAsync();
        }

        public override async Task OnDisconnectedAsync(Exception exception)
        {
            var userConnection = _dbContext.UserConnections.FirstOrDefault(m => m.ConnectionId == Context.ConnectionId);
            if (userConnection != null)
            {
                var userId = userConnection.UserId;

                _dbContext.UserConnections.Remove(userConnection);

                var userSessions = _dbContext.SessionUsers.Where(m => m.UserId == userId).ToList();
                {
                    foreach (var userSession in userSessions)
                    {
                        var session = _dbContext.Sessions.FirstOrDefault(m => m.SessionId == userSession.SessionId);
                        if (session != null)
                        {

                            try
                            {
                                await Clients.GroupExcept(session.SessionId.ToString(), new List<string> { Context.ConnectionId }).RemovePlayer(new Player { UserId = userId }); //send remove player
                            }
                            catch (Exception e)
                            {
                                Console.WriteLine(e);
                            }
                            _dbContext.SessionUsers.Remove(userSession);
                        }

                    }
                }
                await _dbContext.SaveChangesAsync();

                await base.OnDisconnectedAsync(exception);
            }
        }
    }
}